'==============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' @file	an002-pxi.bs2
' @target	PICAXE
'
' uM-FPU Application Note 2 - Long Integer Calculations
' Examples for the PICAXE using an I2C interface.
'
' @author Cam Thompson, Micromega Corporation
' @version V1.0 - April 20, 2005
' @changes
'	V1.0 - April 20, 2005
'	- original version
'
'==============================================================================

'===============================================================================
'-------------------- uM-FPU V2.0 PICAXE definitions ---------------------------
'===============================================================================

symbol	fpuID		= 0xC8	' uM-FPU I2C address

'-------------------- uM-FPU opcodes -------------------------------------------

symbol	SELECTA	= 0x00	' select A register
symbol	SELECTB	= 0x10	' select B register
symbol	FWRITEA	= 0x20	' write to register and select A
symbol	FWRITEB	= 0x30	' write to register and select B
symbol	FREAD		= 0x40	' read from register
symbol	FSET		= 0x50	' A = REG
symbol	LSET		= 0x50	' A = REG


symbol	FADD		= 0x60	' A = A + REG (float)
symbol	FSUB		= 0x70	' A = A - REG (float)
symbol	FMUL		= 0x80	' A = A * REG (float)
symbol	FDIV		= 0x90	' A = A / REG (float)

symbol	LADD		= 0xA0	' A = A + REG (long)
symbol	LSUB		= 0xB0	' A = A - REG (long)
symbol	LMUL		= 0xC0	' A = A * REG (long)
symbol	LDIV		= 0xD0	' A = A / REG (long)

symbol	SQRT		= 0xE0	' A = sqrt(A)	
symbol	LOG		= 0xE1	' A = ln(A)
symbol	LOG10		= 0xE2	' A = log(A)
symbol	EXP		= 0xE3	' A = e ** A
symbol	EXP10		= 0xE4	' A = 10 ** A
symbol	SIN		= 0xE5	' A = sin(A) radians
symbol	COS		= 0xE6	' A = cos(A) radians
symbol	TAN		= 0xE7	' A = tan(A) radians
symbol	FLOOR		= 0xE8	' A = nearest integer <= A
symbol	CEIL		= 0xE9	' A = nearest integer >= A
symbol	ROUND		= 0xEA	' A = nearest integer to A
symbol	NEGATE	= 0xEB	' A = -A
symbol	ABS		= 0xEC	' A = |A|
symbol	INVERSE	= 0xED	' A = 1 / A
symbol	DEGREES	= 0xEE	' A = A / (PI / 180) radians to degrees
symbol	RADIANS	= 0xEF	' A = A * (PI / 180) degreees to radians
symbol	SYNC		= 0xF0	' synchronization
symbol	FLOAT		= 0xF1	' copy A to register 0 and convert to float
symbol	FIX		= 0xF2	' copy A to register 0 and convert to long
symbol	FCOMPARE	= 0xF3	' compare A and B
symbol	LOADBYTE	= 0xF4	' write signed byte to reg 0, convert to float
symbol	LOADUBYTE	= 0xF5	' write unsigned byte to reg 0, convert to float
symbol	LOADWORD	= 0xF6	' write signed word to reg 0, convert to float
symbol	LOADUWORD	= 0xF7	' write unsigned word to reg 0, convert to float
symbol	READSTR	= 0xF8	' read zero terminated string
symbol	ATOF		= 0xF9	' convert ASCII to float, store in A register
symbol	FTOA		= 0xFA	' convert float to ASCII
symbol	ATOL		= 0xFB	' convert ASCII to long, store in A register
symbol	LTOA		= 0xFC	' convert long to ASCII
symbol	FSTATUS	= 0xFD	' get the status of A register
symbol	XOP		= 0xFE	' extended opcode
symbol	NOP		= 0xFF	' nop

symbol	FUNCTION	= 0x00	' (XOP) user functions 0-15
symbol	READBYTE	= 0x90      ' (XOP) read low 8 bits of A (long)
symbol	READWORD	= 0x91      ' (XOP) read low 16 bits of A (long)
symbol	READLONG	= 0x92      ' (XOP) read 32-bit long value from A
symbol	READFLOAT	= 0x93      ' (XOP) read floating point value from A
symbol	LINCA		= 0x94      ' (XOP) A = A + 1 (long)
symbol	LINCB		= 0x95      ' (XOP) A = A + 1 (long)
symbol	LDECA		= 0x96      ' (XOP) A = A - 1 (long)
symbol	LDECB		= 0x97      ' (XOP) A = A - 1 (long)
symbol	LAND		= 0x98      ' (XOP) A = A AND B (long)
symbol	LOR		= 0x99      ' (XOP) A = A OR B (long)
symbol	LXOR		= 0x9A      ' (XOP) A = A XOR B (long)
symbol	LNOT		= 0x9B      ' (XOP) A = NOT A (long)
symbol	LTST		= 0x9C      ' (XOP) status of A AND B (long)
symbol	LSHIFT	= 0x9D      ' (XOP) shift A by B bits (long)
symbol	LWRITEA	= 0xA0	' (XOP) write long to register and select A
symbol	LWRITEB	= 0xB0	' (XOP) write long to register and select B
symbol	LREAD		= 0xC0	' (XOP) read long from register
symbol	LUDIV		= 0xD0	' (XOP) A = A / REG (unsigned long)
symbol	POWER		= 0xE0	' (XOP) A = A ** B
symbol	ROOT		= 0xE1	' (XOP) A = the Bth root of A
symbol	FMIN		= 0xE2	' (XOP) A = minimum of A and B
symbol	FMAX		= 0xE3	' (XOP) A = maximum of A and B
symbol	FRACTION	= 0xE4	' (XOP) load reg 0 with the fractional part of A
symbol	ASIN		= 0xE5	' (XOP) A = asin(A) radians
symbol	ACOS		= 0xE6	' (XOP) A = acos(A) radians
symbol	ATAN		= 0xE7	' (XOP) A = atan(A) radians
symbol	ATAN2		= 0xE8	' (XOP) A = atan(A/B)
symbol	LCOMPARE	= 0xE9	' (XOP) long compare A and B
symbol	LUCOMPARE	= 0xEA	' (XOP) unsigned long compare A and B
symbol	LSTATUS	= 0xEB	' (XOP) long status
symbol	LNEGATE	= 0xEC	' (XOP) A = -A (long)
symbol	LABS		= 0xED	' (XOP) A = |A| (long)
symbol	LEFT		= 0xEE	' (XOP) right parenthesis
symbol	RIGHT		= 0xEF	' (XOP) left parenthesis
symbol	LOADZERO	= 0xF0	' (XOP) load register 0 with zero
symbol	LOADONE	= 0xF1	' (XOP) load register 0 with 1.0
symbol	LOADE		= 0xF2	' (XOP) load register 0 with e
symbol	LOADPI	= 0xF3	' (XOP) load register 0 with pi
symbol	LONGBYTE	= 0xF4	' (XOP) write signed byte to reg 0, convert to long
symbol	LONGUBYTE	= 0xF5	' (XOP) write unsigned byte to reg 0, convert to long
symbol	LONGWORD	= 0xF6	' (XOP) write signed word to reg 0, convert to long
symbol	LONGUWORD	= 0xF7	' (XOP) write unsigned word to reg 0, convert to long
symbol	IEEEMODE	= 0xF8	' (XOP) set IEEE mode (default)
symbol	PICMODE	= 0xF9	' (XOP) set PIC mode
symbol	CHECKSUM	= 0xFA	' (XOP) calculate code checksum
symbol	BREAK		= 0xFB	' (XOP) debug breakpoint
symbol	TRACEOFF	= 0xFC	' (XOP) turn debug trace off
symbol	TRACEON	= 0xFD	' (XOP) turn debug trace on
symbol	TRACESTR	= 0xFE	' (XOP) send debug string to trace buffer
symbol	VERSION	= 0xFF	' (XOP) get version string

symbol	syncChar	= 0x5C      ' sync character 

'-------------------- uM-FPU variables -----------------------------------------

symbol	dataWord	= w6		' (b13,b12) data word
symbol	dataHigh	= b13		' high byte of dataWord
symbol	dataLow	= b12		' low byte of dataWord
symbol	dataByte	= b12		' (alternate name)
symbol	opcode	= b13		' opcode (same as dataHigh)
symbol	opcode2	= b12		' opcode2 (same as dataLow)
symbol	reg		= b13		' register value (same as dataHigh)
symbol	format	= b12		' format (same as dataLow)
symbol	fpu_status	= b12		' status byte (same as dataLow)

'-------------------- uM-FPU status bits ---------------------------------------

symbol	IS_ZERO	= 0x81	' positive zero
symbol	IS_NZERO	= 0x83	' negative zero
symbol	IS_NEGATIVE	= 0x82	' negative
symbol	IS_NAN	= 0x84	' NaN (Not-a-Number)
symbol	IS_PINF	= 0x88	' positive infinity
symbol	IS_NINF	= 0x8A	' negative infinity

'==================== end of uM-FPU definitions ================================

'-------------------- uM-FPU register definitions ----------------------------

symbol	M		= 1		' m value
symbol	N		= 2		' n value
symbol	T		= 3		' t value
symbol	Days		= 4		' number of days
symbol	Hours		= 5		' number of hours
symbol	Minutes	= 6		' number of minutes
symbol	Seconds	= 7		' number of seconds

'-------------------- variables ----------------------------------------------

symbol	i           = b0		' loop counter
symbol	svalue      = w1		' 16-bit sensor value (b2, b3)
symbol	svalueHigh	= b3		' high bute of sensor value
symbol	svalueLow	= b2		' low byte of sensor value

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

reset:
	sertxd(13, 10, "App Note 2")
	sertxd(13, 10, "Long Integer calculations", 13, 10)

	i2cslave fpuID, i2cfast, i2cbyte	' select the uM-FPU I2C device	
	gosub fpu_reset				' reset the uM-FPU
	if fpu_status = SyncChar then reset2
	sertxd (13, 10, "uM-FPU not detected.")
	end

reset2:
	gosub print_version			' display the uM-FPU version number

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

main:
	sertxd (13, 10, 13, 10, "[n = 0]")
	'---------------------------------
	writei2c 0, (N, XOP, LOADZERO, LSET)
	gosub print_N

	sertxd (13, 10, 13, 10, "[n = n + 1]")
	'-------------------------------------
	writei2c 0, (N, XOP, LINCA)
	gosub print_N
	opcode = SELECTB+N
	writei2c 0, (opcode, XOP, LINCB)
	gosub print_N

	sertxd (13, 10, 13, 10, "[n = n - 1]")
	'-------------------------------------
	writei2c 0, (N, XOP, LDECA)
	gosub print_N
	opcode = SELECTB+N
	writei2c 0, (opcode, XOP, LDECB)
	gosub print_N

	' set test values: for m = 500195, n = 100
	sertxd (13, 10)
	writei2c 0, (M, XOP, LWRITEB, $00, $07, $A1, $E3, LSET)
	gosub print_M
	writei2c 0, (N, XOP, LONGBYTE, 100, LSET)
	gosub print_N

	sertxd (13, 10, 13, 10, "[m = m + n]")
	'-------------------------------------
	opcode = LADD+N
	writei2c 0, (M, opcode)
	gosub print_M

	sertxd (13, 10, 13, 10, "[t = m / n]")
	'-------------------------------------
	opcode = LSET+M
	opcode2 = LDIV+N
	writei2c 0, (T, opcode, opcode2)
	gosub print_T

	sertxd (13, 10, 13, 10, "[t = m mod n]")
	'---------------------------------------
	opcode = LSET+M
	opcode2 = LDIV+N
	writei2c 0, (T, opcode, opcode2, LSET)
	gosub print_T

	sertxd (13, 10, 13, 10, "[ n = 5m + 30 ]")
	'-----------------------------------------
	opcode = LMUL+M
	writei2c 0, (N, XOP, LWRITEB, $00, $00, $00, $05, LSET, opcode)
	writei2c 0, (XOP, LWRITEB, $00, $00, $00, $1E, LADD)
	gosub print_N

	sertxd (13, 10, 13, 10, "[m = n / 500000  (using ATOL)]")
	'--------------------------------------------------------
	opcode = LSET+N
	writei2c 0, (M, opcode, ATOL, "500000", 0, LDIV)
	gosub print_M

	sertxd (13, 10, 13, 10, "[m = n / 500000  (using LWRITEB)]")
	'-----------------------------------------------------------
	opcode = LSET+N
	writei2c 0, (M, opcode, XOP, LWRITEB, $00, $07, $A1, $20, LDIV)
	gosub print_M

	' set test value: n = 500
	sertxd (13, 10)
	writei2c 0, (N, XOP, LONGWORD, $01, $F4, LSET)
	gosub print_N

	sertxd (13, 10, 13, 10, "[m = n^2]")
	'-----------------------------------
	opcode = LSET+N
	opcode2 = LMUL+N
	writei2c 0, (M, opcode, opcode2)
	gosub print_M

	sertxd (13, 10, 13, 10, "[n = m / n]")
	'-------------------------------------
	opcode = LSET+M
	opcode2 = LDIV+N
	writei2c 0, (N, XOP, LEFT, opcode, opcode2, XOP, RIGHT, LSET)
  	gosub print_N

	sertxd (13, 10, 13, 10, "[t = (m + n) / 10]")
	'--------------------------------------------
	opcode = LSET+M
	opcode2 = LADD+N
	writei2c 0, (T, opcode, opcode2, XOP, LONGBYTE, 10, LDIV)
	gosub print_T

	sertxd (13, 10, 13, 10, "[t = m / (n + t)]")
	'-------------------------------------------
	opcode = LSET+M
	opcode2 = LSET+N
	writei2c 0, (T, XOP, LEFT, opcode, XOP, LEFT, opcode2)
	opcode = LADD+T
	writei2c 0, (opcode, XOP, RIGHT, LDIV, XOP, RIGHT, LSET)
	gosub print_T

	' set test value: n = 0x12345678 (305419896)
	sertxd (13, 10)                                               
	writei2c 0, (N, XOP, LWRITEB, $12, $34, $56, $78, LSET)
	gosub print_N

	sertxd (13, 10, 13, 10, "[m = bits 20-23 of n]")
	'-----------------------------------------------
	opcode = LSET+N
	opcode2 = -20	
	writei2c 0, (M, opcode, XOP, LONGBYTE, opcode2, XOP, LSHIFT)
	writei2c 0, (XOP, LONGBYTE, 0x0F, XOP, LAND)
	gosub print_M

	sertxd (13, 10, 13, 10, "[extract time values]")
	'-----------------------------------------------
	opcode = LWRITEB+T
	writei2c 0, (XOP, opcode, $00, $07, $A1, $20) ' set time value to 500000
	opcode = LSET+T
	writei2c 0, (Days, opcode, XOP, LWRITEB, $00, $01, $51, $80, XOP, LUDIV) ' divide by 86400
      writei2c 0, (Hours, LSET, XOP, LONGWORD, $0E, $10, LDIV)			' divide by 3600
      writei2c 0, (Minutes, LSET, XOP, LONGBYTE, 60, LDIV)				' divide by 60
      writei2c 0, (Seconds, LSET)
	sertxd (13, 10, "Days = ")
	writei2c 0, (Days)
	gosub print_long
	sertxd (13, 10, "Hours = ")
	writei2c 0, (Hours)
	gosub print_long
	sertxd (13, 10, "Minutes = ")
	writei2c 0, (Minutes)
	gosub print_long
	sertxd (13, 10, "Seconds = ")
	writei2c 0, (Seconds)
	gosub print_long

	' read 100 signed 16-Bit samples from a sensor, and calculate the average
	'------------------------------------------------------------------------
	sertxd (13, 10, 13, 10, "[read samples]")

	writei2c 0, (N, XOP, LOADZERO, LSET)
	for i = 1 to 100
	  gosub read_Sensor
	  gosub fpu_Wait
	  writei2c 0, (XOP, LONGWORD, svalueHigh, svalueLow, LADD)
	next

	writei2c 0, (XOP, LONGBYTE, 100, LDIV)
	gosub print_N

	sertxd (13, 10, 13, 10, "Done.", 13, 10)
	end

'-------------------- read_sensor ---------------------------------------------

read_Sensor:
  svalue = 10000 ' the actual code for reading a sensor would replace this line
  RETURN

'-------------------- print routines ------------------------------------------

print_M:
	sertxd (13, 10, "  m = ")
	gosub print_long
	return

print_N:
	sertxd (13, 10, "  n = ")
	gosub print_long
	return

print_T:
	sertxd (13, 10, "  t = ")
	gosub print_long
	return

'===============================================================================
'-------------------- uM-FPU V2.0 PICAXE support routines ----------------------
'===============================================================================

fpu_reset:
	writei2c 1, (0)			' reset the uM-FPU
	pause 8				' wait for reset to complete
	
	writei2c 0, (SYNC)		' check for synchronization
						' (fall through to fpu_readStatus)
fpu_readStatus:
	readi2c 0, (fpu_status)		' read status byte
	return

fpu_wait:
	readi2c 0, (fpu_status)		' wait for ready status
	if fpu_status <> 0 then fpu_wait
	return

print_version:
	writei2c 0, (XOP, VERSION)	' get the uM-FPU version string
	goto print_string2            ' print it

print_float:
	format = 0				' set format to zero (free format)
						' (fall through to print_floatFormat)
print_floatFormat:
	opcode = FTOA			' convert floating point to formatted ASCII
	goto print_string			' print the value

print_long:
	format = 0				' set format to 0 (free format)
						' (fall through to print_longFormat)
print_longFormat:
	opcode = LTOA			' convert long integer to formatted ASCII
						' (fall through to print_string)
print_string:
	writei2c 0, (opcode, format)	' send conversion command

print_string2:
	gosub fpu_wait			' wait until uM-FPU is ready
	writei2c 0, (READSTR)

print_string3:		
	readi2c 0, (dataByte)		' display zero terminated string
	if dataByte = 0 then Print_String4
	if dataByte > 127 then Print_String4
	sertxd(dataByte)
	goto Print_String3

print_string4:
	return				' end of string

'==================== end of uM-FPU support routines ===========================



